/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2014  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.04a                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for Freescale MPC5645S-DEMO-V2 eval board
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#define SIUL_BASE_ADDR         0xC3F90000u
#define SIUL_PCR70             (*(volatile unsigned short *)(SIUL_BASE_ADDR + 0x040u + (70u * 2u)))  // PF0 Green LED
#define SIUL_PCR121            (*(volatile unsigned short *)(SIUL_BASE_ADDR + 0x040u + (121u * 2u))) // PK0 Red LED
#define GPDO70                 (*(volatile unsigned char  *)(SIUL_BASE_ADDR + 0x600u + 70u))
#define GPDO121                (*(volatile unsigned char  *)(SIUL_BASE_ADDR + 0x600u + 121u))

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  SIUL_PCR70  = 0x200;   // Enable output buffer
  SIUL_PCR121 = 0x200;   
  GPDO70      = 0x01;    // Initially turn off LEDs
  GPDO121     = 0x01;
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if (Index == 0) {
    GPDO70 = 0x00u;   
  } else if (Index == 1) {
    GPDO121 = 0x00u;
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if (Index == 0) {
    GPDO70 = 0x01u;   
  } else if (Index == 1) {
    GPDO121 = 0x01u;
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if (Index == 0) {
    GPDO70 ^= 0x01u;    
  } else if (Index == 1) {
    GPDO121 ^= 0x01u;
  }
}

/****** End Of File *************************************************/
