/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2014  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.04a                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : OS_Syscalls.c
Purpose : sbrk() function for embOS
          The original sbrk() function does not work with embOS tasks 
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "stdlib.h" 

/*********************************************************************
*
*       Extern variables
*
**********************************************************************
*/
// _heap_addr and _heap_end are defined in the linker file.
// Without the "section .data" directive the compiler tries to address 
// these symbols with short addressing (if enabled) which does not work
//
__declspec(section ".data") extern char _heap_addr; 
__declspec(section ".data") extern char _heap_end;

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       sbrk()
*
* Function description
*   Increase program data space. As malloc and related functions depend on this,
*   it is required to have a working implementation.
*   _heap_addr and _heap_end have to be defined in the linker file
*/
void* sbrk(int incr);
void* sbrk(int incr) {
  static char * heap_end;
  char *        prev_heap_end;

  if (heap_end == NULL) {
    heap_end = &_heap_addr;
  }
  prev_heap_end = heap_end;

  if (heap_end + incr > &_heap_end) {
    return (void*) -1;
  }

  heap_end += incr;
  return (void*) prev_heap_end;
}

/****** End Of File *************************************************/
