/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2014  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.04a                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_MPC5645S.c for MPC5645S CPU 
          
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS
          Feel free to modify this file acc. to your target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

/*********************************************************************
*
*       Clock frequency settings
*/
#ifndef   OS_FSYS                   /* CPU Main clock frequency     */
  #define OS_FSYS 124000000uL       /* Assume 124MHz                */
#endif

#ifndef   OS_PCLK_TIMER             /* Peripheral clock for timer   */
  #define OS_PCLK_TIMER (OS_FSYS)   /* May vary from CPU clock      */
#endif                              /* depending on CPU             */

#ifndef   OS_PCLK_UART              /* Peripheral clock for UART    */
  #define OS_PCLK_UART (OS_FSYS)    /* May vary from CPU clock      */
#endif                              /* depending on CPU             */

#ifndef   OS_TICK_FREQ
  #define OS_TICK_FREQ (1000)
#endif

/*********************************************************************
*
*       Configuration of communication to OSView
*/
#ifndef   OS_VIEW_ENABLE            // Global enable of communication
  #define OS_VIEW_ENABLE    (1)     // Default: on
#endif

#ifndef   OS_VIEW_USE_UART          // If set, UART will be used for communication
  #define OS_VIEW_USE_UART  (1)     // Default: 1 => Uart is used
#endif                              // if OS_VIEW_ENABLE is on

/*********************************************************************
*
*       UART settings for embOSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to embOSView, please define it to be -1
*       Currently UART0 is supported and the standard
*       setup enables UART0 per default
*/
#ifndef   OS_UART
  #define OS_UART (0)
#endif

#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE (38400)
#endif

/*********************************************************************
*
*       Interrupt and target configuration
*/
#ifndef   ALLOW_NESTED_INTERRUPTS
  #define ALLOW_NESTED_INTERRUPTS (0)   // Caution: Nesting interrupts will cause higher stack load on C stack 
#endif

/****** End of configurable options *********************************/

#define OS_UART_USED  ((OS_VIEW_ENABLE && (OS_VIEW_USE_UART != 0)) && (OS_UART == 0))

#if OS_UART_USED
  #define OS_COM_INIT() OS_COM_Init()
#else
  #define OS_COM_INIT()
#endif

#define OS_TIMER_RELOAD (OS_PCLK_TIMER / OS_TICK_FREQ)

/*********************************************************************
*
*       Check configuration
*
**********************************************************************
*/

#ifndef   DEBUG     /* Should normally be defined as project option */
  #define DEBUG  (0)
#endif

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/
#define ME_BASE_ADDR           0xC3FDC000u
#define ME_GS                  (*(volatile OS_U32 *)(ME_BASE_ADDR + 0x000u))
#define ME_MCTL                (*(volatile OS_U32 *)(ME_BASE_ADDR + 0x004u))
#define ME_MER                 (*(volatile OS_U32 *)(ME_BASE_ADDR + 0x008u))
#define ME_RUN0_MC             (*(volatile OS_U32 *)(ME_BASE_ADDR + 0x030u))
#define ME_RUN_PC1             (*(volatile OS_U32 *)(ME_BASE_ADDR + 0x084u)) 
#define ME_PCTL_48             (*(volatile OS_U8  *)(ME_BASE_ADDR + 0x0F0u))
#define ME_PCTL_68             (*(volatile OS_U8  *)(ME_BASE_ADDR + 0x104u))
#define ME_PCTL_92             (*(volatile OS_U8  *)(ME_BASE_ADDR + 0x11Cu))
#define ME_S_MTRANS            (0x01u << 27u) // Mode transition status
#define ME_S_CURRENTMODE       (0x0Fu << 28u) // Current device mode status
#define ME_S_CURRENTMODE_SHIFT 28u

#define PIT_BASE_ADDR          0xC3FF0000u
#define PIT_MCR                (*(volatile OS_U32 *)(PIT_BASE_ADDR + 0x00u))
#define PIT_CH0_OFFSET         0x100u
#define PIT0_LDVAL             (*(volatile OS_U32 *)(PIT_BASE_ADDR + PIT_CH0_OFFSET + 0x00u))
#define PIT0_CVAL              (*(volatile OS_U32 *)(PIT_BASE_ADDR + PIT_CH0_OFFSET + 0x04u))
#define PIT0_TCTRL             (*(volatile OS_U32 *)(PIT_BASE_ADDR + PIT_CH0_OFFSET + 0x08u))
#define PIT0_TFLG              (*(volatile OS_U32 *)(PIT_BASE_ADDR + PIT_CH0_OFFSET + 0x0Cu))
#define PIT0_ISR_INDEX         59u
#define PIT0_ISR_PRIO          1u

#define INTC_BASE_ADDR         0xFFF48000u
#define INTC_MCR               (*(volatile OS_U32 *)(INTC_BASE_ADDR + 0x00u))
#define INTC_CPR               (*(volatile OS_U32 *)(INTC_BASE_ADDR + 0x08u))               
#define INTC_IACKR             (*(volatile OS_U32 *)(INTC_BASE_ADDR + 0x10u))
#define INTC_EOIR              (*(volatile OS_U32 *)(INTC_BASE_ADDR + 0x18u))
#define INTC_SSCIR0_3
#define INTC_SSCIR4_7
#define INTC_PSR_ADDR          (INTC_BASE_ADDR + 0x040u)
#define INTC_NUM_INT_SOURCES   238u
#define INTC_NUM_PRIO          16u

#define FMPLL0_BASE_ADDR       0xC3FE00A0u
#define FMPLL0_CR              (*(volatile OS_U32 *)(FMPLL0_BASE_ADDR + 0x00))
#define FMPLL0_MR              (*(volatile OS_U32 *)(FMPLL0_BASE_ADDR + 0x04))

#define OS_BAUDDIVIDE         (((OS_PCLK_UART) / (OS_BAUDRATE * 16uL)))

#define SIUL_BASE_ADDR         0xC3F90000u
#define SIUL_PCR18             (*(volatile unsigned short *)(SIUL_BASE_ADDR + 0x040u + (18u * 2u))) // TXD_0, PB2
#define SIUL_PCR19             (*(volatile unsigned short *)(SIUL_BASE_ADDR + 0x040u + (19u * 2u))) // RXD_0, PB3

#define UART0_BASE_ADDR        0xFFE40000u
#define UART0_LINCR1           (*(volatile unsigned int*) (UART0_BASE_ADDR + 0x00u))
#define UART0_LINIER           (*(volatile unsigned int*) (UART0_BASE_ADDR + 0x04u))
#define UART0_UARTCR           (*(volatile unsigned int*) (UART0_BASE_ADDR + 0x10u))
#define UART0_UARTSR           (*(volatile unsigned int*) (UART0_BASE_ADDR + 0x14u))
#define UART0_LINFBRR          (*(volatile unsigned int*) (UART0_BASE_ADDR + 0x24u))
#define UART0_LINIBRR          (*(volatile unsigned int*) (UART0_BASE_ADDR + 0x28u))
#define UART0_GCR              (*(volatile unsigned int*) (UART0_BASE_ADDR + 0x8Cu))
#define UART0_BDRL3            (*(volatile unsigned char*)(UART0_BASE_ADDR + 0x3Bu))
#define UART0_BDRM3            (*(volatile unsigned char*)(UART0_BASE_ADDR + 0x3Fu))
#define UARTCR_UARTMODE        0x01u
#define UARTCR_TXEN            0x10u
#define UARTCR_RXEN            0x20u
#define UARTCR_WL0             0x02u
#define UARRTSR_DTFTFF         0x02u
#define UARRTSR_DRFRFE         0x04u
#define UARRTSR_RMB            0x200u
#define UARTSR_PE3             0x2000u
#define UARTSR_FEF             0x100u
#define UARTSR_ERR_BITS        UARTSR_PE3 | UARTSR_FEF
#define UARTLINIER_DRIE        0x04
#define UARTLINIER_DTIE        0x02
#define UARTLINCR1_SLEEP       0x02u
#define UARTLINCR1_INIT        0x01u
#define UART0_ISR_PRIO         0x01u
#define UART0_RX_ISR_INDEX     79u        
#define UART0_TX_ISR_INDEX     80u

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/
#pragma push /* Save the current state */
#pragma section data_type ".__initialized_intc_handlertable" ".__uninitialized_intc_handlertable"
/* The INTC vector table will be placed in RAM. 
   We will use the ".__uninitialized_intc_handlertable" name to do the 
   placement in the Linker Command File (.lcf) to avoid the initialization at
   startup time.  This will decrease the code size, but the table won't be 
   initialized to zero.
*/ 
static OS_ISR_HANDLER *_INTCInterruptsHandlerTable[INTC_NUM_INT_SOURCES];
#pragma pop

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _DeFaultIntHandler()
*
* Function description 
*   This handler is initially written into all interrupt vectors.
*   It is called, if no interrupt vector was installed for
*   specific interrupt source.
*   May be used during debugging to detect uninstalled interrupts
*/
static void _DeFaultIntHandler(void) {
  while (1) {
  }
}

/*********************************************************************
*
*       _INTC_Init()
*
* Function description 
*   Initialize the interrupt controller
*/
static void _INTC_Init(void) {
#if DEBUG
  unsigned int i;
#endif
  INTC_IACKR = (OS_U32)_INTCInterruptsHandlerTable; // Set the location of the ISR Handler Table in INTC IACKR Register 
  INTC_MCR   = 0x00;                                // Set INTC_MCR.HVEN = 0 for SW vector mode and INTC_MCR.VTES = 0 for 4-byte branch table entries
  INTC_CPR   = 0x00;                                // Set INTC priority to 0 to allow interrupts with any priority
#if DEBUG
  for (i = 0; i < INTC_NUM_INT_SOURCES; i++) {
    _INTCInterruptsHandlerTable[i] = _DeFaultIntHandler;
  }
#endif
}

/*********************************************************************
*
*       OS_ISR_Tick()
*
* Function description 
*   The OS system timer interrupt
*/
static void OS_ISR_Tick (void);
static void OS_ISR_Tick (void) {
  PIT0_TFLG = 0x01u; // Clear PIT0 flag 
  OS_TICK_Handle();
}

/********************************************************************* 
* 
*       _OS_GetHWTimerCycles()
* 
* Function description 
*   Returns the current hardware timer count value
* 
* Return value 
*   Current timer count value
*/ 
static unsigned int _OS_GetHWTimerCycles(void) {
  return (OS_TIMER_RELOAD - PIT0_CVAL);
}

/********************************************************************* 
* 
*       _OS_GetHWTimerCycles()
* 
* Function description 
*   Returns if the hardware timer interrupt pending flag is set
* 
* Return value 
*   == 0; Interrupt pending flag not set
*   != 0: Interrupt pending flag set
*/ 
static unsigned int _OS_GetHWTimer_IntPending(void) {
  return (PIT0_TFLG & 0x01u);
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

const OS_SYSTIMER_CONFIG SysTimerConfig = {OS_PCLK_TIMER, OS_TICK_FREQ, 0, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};

/*********************************************************************
*
*       OS_InitHW()
*
* Function description 
*   Initialize the hardware (timer) required for the OS to run.
*   May be modified, if an other timer should be used
*/
void OS_InitHW(void) {
  OS_IncDI();
  //
  // Initialize interrupt controller
  //
  _INTC_Init();  
  //
  // Set PLL to 124 MHz and enable clocks for timer
  //
  FMPLL0_CR  = 0x003E0041u;
  ME_MER     = 0x0000001Du;       // Enable DRUN, RUN0, SAFE, RESET modes  
  ME_RUN0_MC = 0x001F0074u;       // RUN0 cfg: 16MHzIRCON,OSC0ON,PLL0ON,syclk=PLL 
  ME_RUN_PC1 = 0x00000010u;     // Peri. Cfg. 1 settings: only run in RUN0 mode 
  ME_PCTL_92 = 0x01u;             // PIT, RTI: select ME_RUN_PC[0]Mode Transition to enter RUN0 mode:
  ME_PCTL_68 = 0x01u;             // SIU
  ME_PCTL_48 = 0x01u;             // Uart
  ME_MCTL    = 0x40005AF0u;       // Enter RUN0 Mode & Key 
  ME_MCTL    = 0x4000A50Fu;       // Enter RUN0 Mode & Inverted Key   
  while (ME_GS & ME_S_MTRANS) {}  // Wait for mode transition to complete    
  while (((ME_GS & ME_S_CURRENTMODE) >> ME_S_CURRENTMODE_SHIFT) != 0x04u) {} // Verify RUN0 is the current mode
  while ((FMPLL0_CR & 0x08) == 0x00u) {} // Wait until PLL is locked
  //
  // Initialize timer for embOS
  //
  PIT_MCR    = 0x01u;             // Enable PIT and configure to stop in debug mode 
  PIT0_LDVAL = OS_TIMER_RELOAD;  
  PIT0_TCTRL = 0x03u;             // Enable PIT1 interrupt & start PIT counting  
  OS_PPC_InstallISRHandler(PIT0_ISR_INDEX, OS_ISR_Tick, PIT0_ISR_PRIO);
  //
  // Setup values for usec precise system time functions
  //
  OS_Config_SysTimer(&SysTimerConfig);
  //
  // Initialize the optional UART for embOSView
  //
  OS_COM_INIT();
  OS_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*       Please note:
*       This is basically the "core" of the idle loop.
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggling an output or incrementing a counter)
*/
void OS_Idle(void) {    // Idle loop: No task is ready to execute
  while (1) {           // Nothing to do ... wait for interrupt
    #if (DEBUG == 0)
                        // Switch CPU into sleep mode
    #endif
  }
}

/*********************************************************************
*
*       OS_GetTime_Cycles()
*
* Function description 
*   This routine is required for task-info via OSView or high
*   resolution time measurement functions.
*   It returns the system time in timer clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U32 Time;
  OS_U32 Cnt;

  Time = (OS_U32)OS_GetTime32();
  Cnt  = OS_TIMER_RELOAD - PIT0_CVAL;
  //
  // Check if timer interrupt pending ...
  //
  if (PIT0_TFLG & 0x01u) {
    Cnt = OS_TIMER_RELOAD - PIT0_CVAL;  // Interrupt pending, re-read timer and adjust result
    Time++;
  }
  return (OS_TIMER_RELOAD * Time) + Cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us()
*
* Function description 
*   Convert Cycles into micro seconds.
*
*   If your clock frequency is not a multiple of 1 MHz,
*   you may have to modify this routine in order to get proper
*   diagnostics.
*
*   This routine is required for profiling or high resolution time
*   measurement only. It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/
#if OS_UART_USED

/*********************************************************************
*
*       _OS_ISR_Rx()
*
* Function description 
*   Interrupt handler for UART reception
*/
static void _OS_ISR_Rx (void) {
  unsigned char c;
  UART0_UARTSR = UARRTSR_DRFRFE;  // Clear data reception completed flag
  UART0_UARTSR = UARRTSR_RMB;     // Clear ready to be read flag
  c = UART0_BDRM3;
  OS_OnRx(c);
}

/*********************************************************************
*
*       _OS_ISR_Tx()
*
* Function description 
*   Interrupt handler for UART transmission
*/
static void _OS_ISR_Tx (void) {
  UART0_UARTSR = UARRTSR_DTFTFF;       // Clear data transmission completed flag  
  if (OS_OnTx()) {                     // No more characters to send?
    UART0_LINIER &= ~UARTLINIER_DTIE;  // Disable further Tx interrupts
  }
}

/*********************************************************************
*
*       OS_COM_Init()
*/
void OS_COM_Init(void) {
  SIUL_PCR18    = 0x0500u;                                     // Select alternative mode 1 -> uart mode
  SIUL_PCR19    = 0x0500u;  
  UART0_LINCR1 &= ~UARTLINCR1_SLEEP;
  UART0_LINCR1 |= UARTLINCR1_INIT;                             // Enable init  
  UART0_UARTCR  = UARTCR_UARTMODE;
  UART0_UARTCR  = UARTCR_UARTMODE | UARTCR_TXEN | UARTCR_RXEN | UARTCR_WL0; // Select 8N1, RX and Tx enable, Uart mode
  UART0_GCR     = 0x00;                                        // One stop bit
  UART0_LINIBRR = OS_BAUDDIVIDE;                               // Set baudrate
  UART0_LINFBRR = (OS_PCLK_UART - (OS_BAUDDIVIDE * 16uL * OS_BAUDRATE)) / OS_BAUDRATE;
  UART0_LINIER |= UARTLINIER_DRIE;                             // Enable Rx interrupts   
  UART0_LINCR1 &= ~UARTLINCR1_INIT;                            // Disable init
  OS_PPC_InstallISRHandler(UART0_RX_ISR_INDEX,  _OS_ISR_Rx,    UART0_ISR_PRIO);
  OS_PPC_InstallISRHandler(UART0_TX_ISR_INDEX,  _OS_ISR_Tx,    UART0_ISR_PRIO);
}

/*********************************************************************
*
*       OS_COM_Send1()
*
* Function description 
*   Send one character via UART.
*   Never call this function from your application
*/
void OS_COM_Send1(OS_U8 c) {
  UART0_LINIER |= UARTLINIER_DTIE;   // Enable Tx interrupts
  UART0_BDRL3 = c;  
}

#else  /* UART for communication not used, define dummy functions */

void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);           /* Avoid compiler warning */
  OS_COM_ClearTxActive();  /* Let the OS know that Tx is not busy */
}
#endif

/*********************************************************************
*
*       OS interrupt handler and ISR specific functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_irq_handler()
*
* Function description 
*   Detect reason for interrupt and call corresponding service routine.
*   OS_irq_handler is called from IRQ_Handler function found in RTOS.s
*/
void OS_irq_handler(void);
void OS_irq_handler(void) {
  OS_ISR_HANDLER *pISR;
  OS_U32 *p;
  
  p    = (OS_U32*)INTC_IACKR;
  pISR = (OS_ISR_HANDLER*) *p;         // Read interrupt vector
  
#if ALLOW_NESTED_INTERRUPTS
  OS_EnterNestableInterrupt();         // Now interrupts may be reenabled. If nesting should be allowed
#else
  OS_EnterInterrupt();                 // Inform OS that interrupt handler is running
#endif
  pISR();                              // Call interrupt service routine
  
  OS_DI();                             // Disable interrupts
  asm("mbar 0");                       // Ensure interrupt flag has finished clearing 
  INTC_EOIR = 0x00;                    // Write 0 to INTC_EOIR, informing INTC to lower priority 
#if ALLOW_NESTED_INTERRUPTS
  OS_LeaveNestableInterrupt();         // Leave nestable interrupt, perform task switch if required
#else
  OS_LeaveInterrupt();                 // Leave interrupt, perform task switch if required
#endif
}

/*********************************************************************
*
*       OS_PPC_InstallISRHandler()
*
* Function description 
*   Installs an interrupt handler
*
* Parameters 
*   ISRIndex:    ISR index
*   pISRHandler: Pointer to handler function
*   Prio:        ISR priority
* 
* Return value
*   Address of previous handler function
*/
OS_ISR_HANDLER* OS_PPC_InstallISRHandler (OS_U32 ISRIndex, OS_ISR_HANDLER* pISRHandler, OS_U8 Prio) {
  OS_ISR_HANDLER *pOldHandler;
  unsigned char  *p;

#if DEBUG
  if (ISRIndex >= INTC_NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
    return NULL;
  }
  if (Prio >= INTC_NUM_PRIO) {
    OS_Error(OS_ERR_ISR_PRIO);
    return NULL;
  }  
#endif
  OS_DI();
  pOldHandler = _INTCInterruptsHandlerTable[ISRIndex];
  _INTCInterruptsHandlerTable[ISRIndex] = pISRHandler;
  p = (OS_U8*)INTC_PSR_ADDR;
  p[ISRIndex] = Prio;
  OS_RestoreI();
  return pOldHandler;
}

/****** End Of File *************************************************/
